﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PdfSharp;
using System.IO;
using PdfSharp.Drawing;
using System.Web;
using PdfSharp.Pdf;

using PdfSharp.Pdf.IO;
using PdfSharp.Drawing.Layout;
using DHTMLX.Export.PDF.Formatter;
using DHTMLX.Export.PDF.Scheduler;

namespace DHTMLX.Export.PDF
{
    public class SchedulerPDFWriter
    {
        public string HeaderImgPath = "header.png";
        public string FooterImgPath = "footer.png";
        public double HeaderImgHeight = 100;
        public double FooterImgHeight = 100;
        public double BaseFontSize = 7.0;
        public double WeekAgendarDayFont = 9.4;

        public bool TrimContentStrings = true;
        public bool TrimLayoutStrings = true;

        public double HeaderFontSize = 7.0;

        protected XMLParser parser;
        protected PdfDocument pdf;
        protected PdfPage page;
        protected List<PdfPage> pages;
        protected List<XGraphics> graphics;
        protected XGraphics gfx;
        protected XFont f1 = null;
        protected PDFImages images = new PDFImages();
        public string ContentType { get { return "application/pdf"; } }
	    private double pageWidth = 0;
	    private double pageHeight = 0;
	    private double offsetTop = 30;
	    private double offsetBottom = 30;
	    private double offsetLeft = 30;
	    private double offsetRight = 30;
	    private double headHeight = 20;
	    private double monthDayHeaderHeight = 20;
	    private double monthEventHeight = 12;
	    private double leftScaleWidth = 80;
	    private double timelineLeft = 80;
	    private double monthEventOffsetLeft = 2;
	    private double monthEventOffsetTop = 2;
	    private double yearMonthOffsetLeft = 10;
	    private double yearMonthOffsetTop = 10;
	    private double yearMonthLabelHeight = 20;
	    private double weekEventHeaderHeight = 10;
	    private double agendaColOneWidth = 150;
	    private double multidayLineHeight = 14;
	    private double weekAgendaEventHeight = 20;
        public double BorderWidth = 0.5;
	    private string linePattern = "0";

	    private string pageNumTemplate = "Page {pageNum}/{allNum}";
        protected bool _NeedPagesNums = false;

        protected PDFPage _CurrentPage { get; set; }

	    private double monthDayWidth;
	    private double monthDayHeight;
	    private double contHeight;
	    private double contWidth;

	    private double weekDayWidth;
	    private double weekDayHeight;
	    private double headerHeight;
	    private double multiHeight = 0;

        private ColorProfile profile;

	    private double cellOffset = 3;

	    private string bgColor = "C2D5FC";
	    private string lineColor = "586A7E";
	    private string headerLineColor = "FFFFFF";

	    private string dayHeaderColor = "EBEFF4";
	    private string watermarkTextColor = "8b8b8b";
	    private string dayBodyColor = "FFFFFF";
	    private string dayHeaderColorInactive = "E2E3E6";
	    private string dayBodyColorInactive = "ECECEC";
	    private string eventColor = "FFE763";
	    private string eventTextColor = "887A2E";
	    private string scaleOneColor = "FCFEFC";
	    private string scaleTwoColor = "DCE6F4";
        private string timelineTreeColor = "969394";
	    private string eventBorderColor = "B7A543";
	    private string textColor = "000000";
	    private string yearDayActiveColor = "EBEFF4";
	    private string yearDayInactiveColor = "D6D6D6";
	    private string multidayColor = "E1E6FF";
	    private string matrixEventColor = "FFFFFF";
	    private string view;
	    private string watermark = null;

        private double[] selectColor(string fullColor, double[] def){
            if ((fullColor != "transparent")
                        && ((this.profile == ColorProfile.FullColor) || (this.profile == ColorProfile.Color)))
            {
				    return RGBColor.GetColor(fullColor);
			    } else {
				    return def;
			    }
        }

        private string textWrap(string text, double width, XFont f, bool actuallyDo)
        {
            if (!actuallyDo)
                return text;
            else
                return textWrap(text, width, f);
            
        }

        private string textWrap(string text, double width, XFont f) {
		    if ((gfx.MeasureString(text, f).Width <= width)||(text.Length == 0)) {
			    return text;
		    }
		    while ((gfx.MeasureString(text + "...", f).Width  > width)&&(text.Length > 0)) {
			    text = text.Substring(0, text.Length - 1);
		    }
		    return text + "...";
	    }
        public double[] GetSizes(PageOrientation orientation, PageSize size)
        {
            var sizes = new double[2];
            var siz= PageSizeConverter.ToSize(size);

            if (orientation == PageOrientation.Landscape)
            {
                    
                sizes[0] = siz.Height;
                sizes[1] = siz.Width;
            }
            else
            {
                sizes[1] = siz.Height;
                sizes[0] = siz.Width;
            }

            return sizes;
            
        }
        private void setColorProfile(ColorProfile profile)
        {
            if ((profile == ColorProfile.Color) || (profile == ColorProfile.FullColor))
            {
			    this.bgColor = "C2D5FC";
			    this.lineColor = "c1d4fc";
			    this.headerLineColor = "FFFFFF";
			    this.dayHeaderColor = "EBEFF4";
			    this.dayBodyColor = "FFFFFF";
			    this.dayHeaderColorInactive = "E2E3E6";
			    this.dayBodyColorInactive = "ECECEC";
			    this.eventColor = "FFE763";
			    this.eventTextColor = "887A2E";
			    this.scaleOneColor = "FCFEFC";
			    this.scaleTwoColor = "DCE6F4";

                this.timelineTreeColor = "969394";

			    this.eventBorderColor = "B7A543";
			    this.textColor = "000000";
			    this.yearDayActiveColor = "EBEFF4";
			    this.yearDayInactiveColor = "D6D6D6";
			    this.multidayColor = "E1E6FF";
			    this.matrixEventColor = "FFFFFF";
			    this.watermarkTextColor = "8b8b8b";
		    } else {
			    if (profile == ColorProfile.Gray) {
				    this.bgColor = "D3D3D3";
				    this.lineColor = "666666";
				    this.headerLineColor = "FFFFFF";
				    this.dayHeaderColor = "EEEEEE";
				    this.dayBodyColor = "FFFFFF";
				    this.dayHeaderColorInactive = "E3E3E3";
				    this.dayBodyColorInactive = "ECECEC";
				    this.eventColor = "DFDFDF";
				    this.eventTextColor = "000000";
				    this.scaleOneColor = "FFFFFF";

                    this.timelineTreeColor = "BBBBBB";

				    this.scaleTwoColor = "E4E4E4";
				    this.eventBorderColor = "9F9F9F";
				    this.textColor = "000000";
				    this.yearDayActiveColor = "EBEFF4";
				    this.yearDayInactiveColor = "E2E3E6";
				    this.multidayColor = "E7E7E7";
				    this.matrixEventColor = "FFFFFF";
				    this.watermarkTextColor = "8b8b8b";
			    } else {
				    this.bgColor = "FFFFFF";
				    this.lineColor = "000000";
				    this.headerLineColor = "000000";
				    this.dayHeaderColor = "FFFFFF";
				    this.dayBodyColor = "FFFFFF";
				    this.dayHeaderColorInactive = "FFFFFF";
				    this.dayBodyColorInactive = "FFFFFF";
				    this.eventColor = "FFFFFF";
				    this.eventTextColor = "000000";

                    this.timelineTreeColor = "FFFFFF";

				    this.scaleOneColor = "FFFFFF";
				    this.scaleTwoColor = "FFFFFF";
				    this.eventBorderColor = "000000";
				    this.textColor = "000000";
				    this.yearDayActiveColor = "FFFFFF";
				    this.yearDayInactiveColor = "FFFFFF";
				    this.multidayColor = "FFFFFF";
				    this.matrixEventColor = "FFFFFF";
				    this.watermarkTextColor = "000000";
			    }
		    }
	    }
        private void createPDF(PageOrientation orientation, PageSize size)
        {
            if (this.pdf == default(PdfDocument))
            {
                pdf = new PdfDocument();
                var font_settings = new XPdfFontOptions(PdfFontEncoding.Unicode);
                f1 = createFont("Helvetica", 10);
                pdf.Version = 14;
                pages = new List<PdfPage>();
            }
            newPage(orientation, size);

            double[] sizes = GetSizes(orientation, size);
            pageWidth = sizes[0] - this.offsetLeft - this.offsetRight;
            pageHeight = sizes[1] - this.offsetTop - this.offsetBottom;

        }
        protected PdfPage newPage(PageOrientation orient, PageSize size)
        {
            page = new PdfPage(pdf);
            pdf.AddPage(page);

            gfx = XGraphics.FromPdfPage(page, XGraphicsPdfPageOptions.Replace);
            if (graphics == null)
                graphics = new List<XGraphics>();
            graphics.Add(gfx);
            pages.Add(page);

            page.Size = size;
            page.Orientation = orient;

            todayLabelDraw(gfx); 

            return page;
        }
        protected XFont createFont(string name, double size)
        {
            var font_settings = new XPdfFontOptions(PdfFontEncoding.Unicode);
            return new XFont(name, size, new XFontStyle(), font_settings);
      
        }


        public MemoryStream Generate(string xml)
        {
            var data = new MemoryStream();

            Generate(xml, data);
            return data;
        }
        public void Generate(string xml, HttpResponse resp)
        {
            var data = new MemoryStream();

            resp.ContentType = ContentType;
            resp.HeaderEncoding = Encoding.UTF8;
            
            resp.AppendHeader("Cache-Control", "max-age=0");
            Generate(xml, data);

            data.WriteTo(resp.OutputStream);


        }
        public void PrintPage(PDFPage page)
        {
            view = page.Mode;
            this.profile = page.Profile;
            this.setColorProfile(this.profile);


            _CurrentPage = page;
            switch (view)
            {
                case "month":
                    {
                        this.printMonth(page);
                        break;
                    }
                case "year":
                    {
                        this.printYear(page);
                        break;
                    }
                case "agenda":
                case "map":
                    {
                        this.printAgenda(page);
                        break;
                    }
                case "treetimeline":
                case "timeline":
                    {
                        this.printTimeline(page);
                        break;
                    }
                case "matrix":
                    {
                        this.printMatrix(page);
                        break;
                    }
                case "week_agenda":
                    {
                        this.printWeekAgenda(page);
                        break;
                    }
                default:
                    {
                        this.printWeek(page);
                        break;
                    }
            }

            //this.todayLabelDraw(this.graphics[this.graphics.Count -1]);
            printWatermark();
        }

        public void PrintPages()
        {
            
            foreach(var i in this.parser.Pages){
                PrintPage(i);
            }
            if(this._NeedPagesNums){
                this.agendaPagesDraw();
            }

        }
	    public void Generate(String xml, Stream resp){
		    this.parser = new XMLParser();
		    this.headerHeight = this.headHeight;
		    try {
			    this.parser.SetXML(xml);
                PrintPages();			    
			    this.outputPDF(resp);               
		    }catch(Exception e){
            }
	    }
        private void outputPDF(Stream resp)
        {
            pdf.Save(resp, false);
        }
        private void printMonth(PDFPage page)
        {
		    this.createPDF(PageOrientation.Landscape, PageSize.A4);
		    this.printHeader();
		    this.printFooter();
		    this.monthHeaderDraw();
		    this.monthContainerDraw();
            this.monthEventsDraw(page);
		    
	    }

        private void printYear(PDFPage page)
        {
		    this.createPDF(PageOrientation.Landscape, PageSize.A4);
		    this.printHeader();
		    this.printFooter();
            this.yearDraw(page);
	    }

        private void printAgenda(PDFPage page)
        {
		    this.createPDF(PageOrientation.Portrait, PageSize.A4);
		    this.printHeader();
		    this.printFooter();
		    this.agendaHeaderDraw();
            this.agendaEventDraw(page);
            this._NeedPagesNums = true;
		    //this.agendaPagesDraw();
	    }

        private void printTimeline(PDFPage page)
        {
		    this.leftScaleWidth = this.timelineLeft;
		    this.createPDF(PageOrientation.Landscape, PageSize.A4);
		    this.printHeader();
		    this.printFooter( );
		    this.weekHeaderDraw( );
		    this.timelineContainerDraw( );
            this.timelineEventsDraw(page);
	    }

        private void printMatrix(PDFPage page)
        {
		    this.leftScaleWidth = this.timelineLeft;
		    this.createPDF(PageOrientation.Landscape, PageSize.A4);
		    this.printHeader( );
		    this.printFooter( );
		    this.weekHeaderDraw( );
            this.matrixContainerDraw(page);
	    }

        private void printWeek(PDFPage page)
        {
		    this.createPDF(PageOrientation.Portrait, PageSize.A4);
		    this.printHeader( );
		    this.printFooter( );
		    this.weekHeaderDraw( );
            this.weekMultidayDraw(page);
		    this.weekContainerDraw( );
            this.weekEventsDraw(page);
		    if (this.profile == ColorProfile.BW)
			    this.weekBwBordersDraw();
	    }

        private void printWeekAgenda(PDFPage page)
        {
		    this.createPDF(PageOrientation.Portrait, PageSize.A4);
		    this.printHeader();
		    this.printFooter();
		    SchedulerEvent[] events = page.Events.ToArray();
            this.weekAgendaContainerDraw();
		    while (events.Length > 0) {
			    
			    events = this.weekAgendaEventsDraw(events);
			    if (events.Length > 0) {
				    this.page = newPage(PageOrientation.Portrait, PageSize.A4);
                    this.weekAgendaContainerDraw();
				    //this.todayLabelDraw(this.graphics[this.graphics.Count -1]);//last added is current
			    }
		    }
	    }

	    private void printWatermark() {
		    if (watermark == null) return;
		
		    f1 = createFont(f1.FontFamily.Name, 10);

            for (var i = 1; i <= pages.Count; i++)
            {
                var page = pages[i - 1];
                var graph = graphics[i - 1];
                
                XTextFormatter text = new XTextFormatter(graph);
                double x = offsetLeft;
                double y = pageHeight + offsetTop + f1.Size;
                text.DrawString(watermark, f1, new XSolidBrush(RGBColor.GetXColor(watermarkTextColor)), new XRect(x, y, graph.MeasureString(watermark, f1).Width, f1.Size));

            }

           
        }
	
	    

	    private void monthHeaderDraw() {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.headerLineColor);
            String[][] cols = this.parser.MonthColsParsing(_CurrentPage);
		    double width = this.pageWidth / cols[0].Length;
		    double height = this.headerHeight;
		    double x = this.offsetLeft;
		    double y = this.offsetTop;

            var font = new XFont(f1.FontFamily.Name, f1.Size);
           
		    for (int i = 0; i < cols[0].Length; i++) {
                XRect cell = new XRect(x, y, width, height);

			    gfx.DrawRectangle(new XSolidBrush(RGBColor.GetXColor(bgColor)), cell);
			 
			    if (i > 0) {
                    var points = new XPoint[4];
                    points[0] = new XPoint(x,y);
                    points[1] = new XPoint(x, y + height);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), 1), points[0], points[1]);
			    }

                XTextFormatter text = new XTextFormatter(this.gfx);
                var content = this.textWrap(cols[0][i], width - 2 * this.cellOffset, font, this.TrimLayoutStrings);
		        double text_x = x+ (width - gfx.MeasureString(content, f1).Width) / 2;
		        double text_y = y+(height - this.f1.Size) / 2;

                text.DrawString(content, font, new XSolidBrush(RGBColor.GetXColor(RGBColor.GetColor(this.textColor))), new XRect(text_x, text_y, gfx.MeasureString(content, f1).Width, font.Size));

			    x += width;
		    }
	    }

	    private void monthContainerDraw() {
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] dayHeaderColor = RGBColor.GetColor(this.dayHeaderColor);
		    double[] dayBodyColor = RGBColor.GetColor(this.dayBodyColor);
		    double[] dayHeaderColorInactive = RGBColor
				    .GetColor(this.dayHeaderColorInactive);
		    double[] dayBodyColorInactive = RGBColor
				    .GetColor(this.dayBodyColorInactive);
            String[,] rows = this.parser.MonthRowsParsing(_CurrentPage);
		    double width = this.pageWidth / 7;
		    double height = (this.pageHeight - this.headerHeight) / rows.GetLength(0);
		    this.monthDayWidth = width;
		    this.monthDayHeight = height;
		    double x = this.offsetLeft;
		    double y = this.offsetTop + this.headerHeight;
            for (int i = 0; i < rows.GetLength(0); i++)
            {

			    for (int j = 0; j < rows.GetLength(1); j++) {
				    bool activeDay = this.getActiveDay(rows, i, j);

                    XRect cell = new XRect(x, y, width, height);  
                    XRect cellIn = new XRect(x, y, width, this.monthDayHeaderHeight);
                    XBrush cellInBrush, cellBrush;
				    if (activeDay == true) {
                        cellBrush = new XSolidBrush(RGBColor.GetXColor(dayBodyColor));
					    cellInBrush = new XSolidBrush(RGBColor.GetXColor(dayHeaderColor));
				    } else {
                        cellBrush = new XSolidBrush(RGBColor.GetXColor(dayBodyColorInactive));
					    cellInBrush = new XSolidBrush(RGBColor.GetXColor(dayHeaderColorInactive));
				    }


                    gfx.DrawRectangle(cellBrush, cell);
                    gfx.DrawRectangle(cellInBrush, cellIn);

                    var points = new XPoint[3];
                    points[0] = new XPoint(x, y);
                    points[1] = new XPoint(x + width, y);
                    points[2] = new XPoint(x, y + height);
			        gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[0], points[2]);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[0], points[1]);
				


                    string label = rows[i,j];
                    double text_x = x + width -gfx.MeasureString(label, f1).Width - this.cellOffset;

                    double text_y = y + (this.monthDayHeaderHeight + f1.Size) / 2;

                    gfx.DrawString(label, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(text_x, text_y));
                    x += width;
			    }
                x = this.offsetLeft;
                
                y += height;
		    }


            var points2 = new XPoint[3];
            points2[0] = new XPoint(this.offsetLeft + this.pageWidth, this.offsetTop);
            points2[1] = new XPoint(this.offsetLeft + this.pageWidth, this.offsetTop + this.pageHeight);
            points2[2] = new XPoint(this.offsetLeft, this.offsetTop + this.pageHeight);

            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points2[0], points2[1]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points2[1], points2[2]);

		  
	    }

        private void monthEventsDraw(PDFPage page)
        {
		    double[] eventColor = RGBColor.GetColor(this.eventColor);
		    double[] eventBorderColor = RGBColor.GetColor(this.eventBorderColor);
		    double[] textColor = RGBColor.GetColor(this.eventTextColor);
		    var events = page.Events;
		    int footenote = 1;
		    int[][] events_grid = new int[6][];
		    for (int i = 0; i < 6; i++) {
			    events_grid[i] = new int[7];
			    for (int j = 0; j < 7; j++) {
				    events_grid[i][j] = 0;
			    }
		    }
		    for (int i = 0; i < events.Count; i++) {
			    string event_text = events[i].getText();
			    int day = events[i].getDay();
			    int week = events[i].getWeek() - 1;

			    double width = (events[i].getWidth() * this.monthDayWidth) / 100;
			    string type = events[i].getType();
			    string bgColor = events[i].getBackgroundColor();
			    string color = events[i].getColor();
			    if (type == "event_line") {

				    double cell_x = this.offsetLeft + day * this.monthDayWidth
						    + this.monthEventOffsetLeft;
				    double cell_y = this.offsetTop;
				    cell_y += this.headerHeight;
				    cell_y += week * this.monthDayHeight;
				    cell_y += this.monthDayHeaderHeight;
				    cell_y += events_grid[week][day]
						    * (this.monthEventHeight + this.monthEventOffsetTop);
				    cell_y += this.monthEventOffsetTop;
                    
                  
                    double height = this.monthEventHeight;
                    XRect cell = new XRect(cell_x, cell_y, width, height);
                    XPen cellPen = new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth);

                    var borders = new XPoint[4];
                    borders[0] = new XPoint(cell_x, cell_y);
                    borders[1] = new XPoint(cell_x + width, cell_y);
                    borders[2] = new XPoint(cell_x + width, cell_y + height);
                    borders[3] = new XPoint(cell_x, cell_y + height);
                   
                   

                    XRect cellIn = new XRect(cell_x, cell_y, width, height);
                    XBrush cellInBrush= new XSolidBrush(RGBColor.GetXColor(selectColor(bgColor, eventColor)));

				    f1 = createFont(f1.FontFamily.Name, 8);



                    string label = this.textWrap(event_text, width - 2 * this.cellOffset, this.f1, this.TrimContentStrings);
                    double text_x = cell_x + this.cellOffset;;

                    double text_y = cell_y + (height + this.f1.Size) / 2;
                   
                    XSolidBrush textBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));

				   

				    if ((cell_y + height) >= (this.offsetTop + this.headerHeight + (week + 1)
						    * this.monthDayHeight)) {
					    int event_footenote = getFootenoteExists(events, week, day);
					    if (event_footenote > 0) {
						    events[i].setFootenote(event_footenote);
					    } else {
						    events[i].setFootenote(footenote);
						    footenote++;
					    }
				    } else {
                        
                        gfx.DrawRectangle(cellInBrush, cellIn);
                        gfx.DrawLine(cellPen, borders[0], borders[1]);
                        gfx.DrawLine(cellPen, borders[1], borders[2]);
                        gfx.DrawLine(cellPen, borders[2], borders[3]);
                        gfx.DrawLine(cellPen, borders[3], borders[0]);

					    gfx.DrawString(label, f1, textBrush, new XPoint(text_x, text_y));
				    }
				    events_grid[week][day]++;
				    int event_length = (int) Math.Floor(width / this.monthDayWidth);
				    for (int j = 1; j <= event_length; j++) {
					    events_grid[week][day + j]++;
				    }
			    } else {
				    double cell_x = this.offsetLeft + day * this.monthDayWidth
						    + this.monthEventOffsetLeft;
				    double cell_y = this.offsetTop;
				    cell_y += this.headerHeight;
				    cell_y += week * this.monthDayHeight;
				    cell_y += this.monthDayHeaderHeight;
				    cell_y += events_grid[week][day]
						    * (this.monthEventHeight + this.monthEventOffsetTop);
				    cell_y += this.monthEventOffsetTop;
				    double height = this.monthEventHeight;
				    f1 = createFont(f1.FontFamily.Name, 8);

                    string text = this.textWrap(event_text, width - 2 * this.cellOffset, this.f1, this.TrimContentStrings);
				    
                    XSolidBrush textBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));
				    


				    double text_x = cell_x + this.cellOffset;
				    double text_y = cell_y + (height + this.f1.Size) / 2;

				   
				    if ((cell_y + height) >= (this.offsetTop + this.headerHeight + (week + 1)
						    * this.monthDayHeight)) {
					    int event_footenote = getFootenoteExists(events, week, day);
					    if (event_footenote > 0) {
						    events[i].setFootenote(event_footenote);
					    } else {
						    events[i].setFootenote(footenote);
						    footenote++;
					    }
				    } else {
                        gfx.DrawString(text, f1, textBrush, new XPoint(text_x, text_y));
					   
				    }
				    events_grid[week][day]++;
			    }
		    }
		    if (this.profile == ColorProfile.BW)
			    this.monthBwBordersDraw();
		    this.monthFootenotesDraw(events, footenote);
	    }

	    private int getFootenoteExists(IList<SchedulerEvent> events, int week, int day) {
		    int result = -1;
		    for (int i = 0; i < events.Count; i++) {
			    int event_week = events[i].getWeek() - 1;
			    int event_day = events[i].getDay();
			    if ((event_week == week) && (event_day == day)) {
				    int foote = events[i].getFootenote();
				    if (foote > 0) {
					    result = events[i].getFootenote();
				    }
			    }
		    }
		    return result;
	    }

	    private void monthFootenotesDraw(IList<SchedulerEvent> events, int footenotes) {
            String[,] rows = this.parser.MonthRowsParsing(_CurrentPage);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] eventColor = RGBColor.GetColor(this.eventColor);
		    double[] eventTextColor = RGBColor.GetColor(this.eventTextColor);
		    if (footenotes > 1) {
                this.page = newPage(PageOrientation.Landscape, PageSize.A4);
			    
		    }
		    double labelHeight = 20;
		    double height = 20;
		    double width = 160;
		    double offsetLeft = 20;
		    double x = this.offsetLeft;
		    double y = this.offsetTop;
		    for (int i = 1; i < footenotes; i++) {
			    var foot_events = this.getFootenoteEvents(events, i);
			    int day = foot_events[0].getDay();
			    int week = foot_events[0].getWeek() - 1;
			    if (y + labelHeight + height > this.offsetTop + this.pageHeight) {
				    x += width + offsetLeft;
				    if (x + width > this.offsetLeft + this.pageWidth) {
                        this.page = newPage(PageOrientation.Landscape, PageSize.A4);
					    
					    x = this.offsetLeft;
				    }
				    y = this.offsetTop;
			    }
			    string label = rows[week,day] + "[" + i.ToString() + "]";


			    double label_x = x + (width - gfx.MeasureString(label, f1).Width) / 2;
			    double label_y = y + labelHeight - this.cellOffset;
                gfx.DrawString(label, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(label_x, label_y));

			    y += labelHeight;
			    for (int j = 0; j < foot_events.Count; j++) {
				    if (y + height > this.offsetTop + this.pageHeight) {
					    x += width + offsetLeft;
					    if (x + width > this.offsetLeft + this.pageWidth) {
						    this.page = newPage(PageOrientation.Landscape, PageSize.A4);
						    x = this.offsetLeft;
					    }
					    y = this.offsetTop;
					    label_x = x + (width - gfx.MeasureString(label, f1).Width) / 2;
					    label_y = y + labelHeight - this.cellOffset;

                        gfx.DrawString(label, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(label_x, label_y));
					   
					    y += labelHeight;
				    }

                  

                    var borders = new XPoint[4];
                    borders[0] = new XPoint(x, y);
                    borders[1] = new XPoint(x + width, y);
                    borders[2] = new XPoint(x + width, y + height);
                    borders[3] = new XPoint(x, y + height);
                    XBrush contBrush =new XSolidBrush(RGBColor.GetXColor(borderColor));
                 
                    XRect contBg = new XRect(x, y, width, height);
				    XBrush evBrush =new XSolidBrush(RGBColor.GetXColor(eventColor));
				  
				    
                    gfx.DrawRectangle(evBrush, contBg);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[0], borders[1]);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[1], borders[2]);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[2], borders[3]);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[3], borders[0]);
				   

				    string text = this.textWrap(foot_events[j].getText(), width - 2 * this.cellOffset, this.f1, this.TrimContentStrings);

                    
				    double text_x = x+ this.cellOffset;
				    double text_y =  y+(height + this.f1.Size) / 2;

                    gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(eventTextColor)), new XPoint(text_x, text_y));
				   
				   

				    y += height;
			    }
		    }
		    if (footenotes > 1) {
                this._NeedPagesNums = true;
		    }
	    }

        private IList<SchedulerEvent> getFootenoteEvents(IList<SchedulerEvent> events,
			    int footenote) {
            IList<SchedulerEvent> foot_events = null;
		    int nums = 0;
		    for (int i = 0; i < events.Count; i++) {
			    if (events[i].getFootenote() == footenote) {
				    nums++;
			    }
		    }
		    foot_events = new List<SchedulerEvent>(nums);
		   // nums = 0;
		    for (int i = 0; i < events.Count; i++) {
			    if (events[i].getFootenote() == footenote) {
				    //foot_events[nums] = events[i];
                    foot_events.Add(events[i]);
				    //nums++;
			    }
		    }
		    return foot_events;
	    }

	    private void monthBwBordersDraw() {
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double x = this.offsetLeft;
		    double y = this.offsetTop;
		    this.Line(borderColor, x, y, x + this.pageWidth, y);
		    this.Line(borderColor, x, y, x, y + this.headerHeight);

		    x = this.offsetLeft + this.pageWidth;
		    this.Line(borderColor, x, y, x, y + this.headerHeight);
	    }

	    private void weekHeaderDraw() {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.headerLineColor);
            String[][] cols = this.parser.WeekColsParsing(_CurrentPage);
		   // double width = (this.pageWidth - this.leftScaleWidth) / cols[0].Length;
		    
            double x, y, height, width;
            this.weekDayWidth = (this.pageWidth - this.leftScaleWidth) / cols[0].Length;
            var defHeight = this.headerHeight;
            this.headerHeight = defHeight*cols.Length;
		    for (int scale = 0; scale < cols.Length; scale++) {
                x = this.offsetLeft + this.leftScaleWidth;
                height = defHeight;
                
                y = this.offsetTop + defHeight*scale;
                width = (this.pageWidth - this.leftScaleWidth) / cols[scale].Length;
                if (this.weekDayWidth > width)
                    this.weekDayWidth = width;
                for (int col = 0; col < cols[scale].Length; col++)
                {

                    XRect cell = new XRect(x, y, width, height);

                    XBrush cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));
                    gfx.DrawRectangle(cellBrush, cell);

                    var points = new XPoint[3];
                    points[0] = new XPoint(x, y);
                    points[1] = new XPoint(x + width, y);
                    points[2] = new XPoint(x, y + height);

                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[0], points[2]);
                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[0], points[1]);


                    this.f1 = new XFont(f1.FontFamily.Name, HeaderFontSize);

                    string text = this.textWrap(cols[scale][col], width
                            - 2 * this.cellOffset, this.f1, this.TrimLayoutStrings);

                    double text_x = x + (width - gfx.MeasureString(text, f1).Width) / 2;
                    double text_y = y + (height + this.f1.Size) / 2;
                    gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(text_x, text_y));

                    x += width;
                }
               
		    }
	    }

        private void weekMultidayDraw(PDFPage page)
        {
		    double[] bgColor = RGBColor.GetColor(this.multidayColor);
		    double[] borderColor = RGBColor.GetColor(this.headerLineColor);

		    multiHeight = this.getMultidayHeight(page);
            var x = this.offsetLeft;
            var y = this.offsetTop + this.headerHeight;
            XRect cell = new XRect(x, y, this.pageWidth, multiHeight);

            XBrush cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));
            gfx.DrawRectangle(cellBrush, cell);
		   
            var points = new XPoint[4];
                points[0] = new XPoint(x + this.leftScaleWidth, y + 0);
                points[1] = new XPoint(x + this.leftScaleWidth, y + multiHeight);
                points[2] = new XPoint(x + 0, y + 0);
                points[3] = new XPoint(x + this.pageWidth, y + 0);

            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[0], points[1]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[2], points[3]);

		    this.headerHeight += multiHeight;
	    }

        private double getMultidayHeight(PDFPage page)
        {
            String[][] cols = this.parser.WeekColsParsing(page);
		    var events = page.Multiday;
		    int[] scheme = new int[cols[0].Length];
		    for (int i = 0; i < scheme.Length; i++)
			    scheme[i] = 0;
		    for (int i = 0; i < events.Count; i++) {
			    int day = events[i].getDay();
			    int len = events[i].getLen();
			    for (int j = day; j < day + len; j++)
				    scheme[j]++;
		    }
		    int max = scheme[0];
		    for (int i = 1; i < scheme.Length; i++) {
			    if (scheme[i] > max)
				    max = scheme[i];
		    }
		    return max * this.multidayLineHeight;
	    }

	    private void weekContainerDraw() {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
		    double[] scaleOneColor = RGBColor.GetColor(this.scaleOneColor);
		    double[] scaleTwoColor = RGBColor.GetColor(this.scaleTwoColor);
		    String[] rows = this.parser.WeekRowsParsing(_CurrentPage);
		    double width = this.leftScaleWidth;
		    double height = (this.pageHeight - this.headerHeight) / rows.Length;
		    this.weekDayHeight = height;
		    double x = this.offsetLeft;
		    double y = this.offsetTop + this.headerHeight;
		    for (int i = 0; i < rows.Length; i++) {

                XRect cell = new XRect(x, y, width, height);
                XBrush cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));
            
                XRect scaleOne = new XRect(x + width, y, this.pageWidth - width, height / 2);
                XBrush scaleOneBrush = new XSolidBrush(RGBColor.GetXColor(scaleOneColor));

                XRect scaleTwo = new XRect(x + width, y + height / 2, this.pageWidth - width, height / 2);
                XBrush scaleTwoBrush = new XSolidBrush(RGBColor.GetXColor(scaleTwoColor));

                gfx.DrawRectangle(cellBrush, cell);
                gfx.DrawRectangle(scaleOneBrush, scaleOne);
                gfx.DrawRectangle(scaleTwoBrush, scaleTwo);
			    
                gfx.DrawLine(new XPen(RGBColor.GetXColor(headerLineColor), BorderWidth), new XPoint(x,y), new XPoint(x+width, y));
			    
                string text = this.textWrap(rows[i], width - 2 * this.cellOffset, this.f1, this.TrimLayoutStrings);
			    double text_x = x + (width - gfx.MeasureString(text, this.f1).Width) / 2;
			    double text_y = y + (height + this.f1.Size) / 2;

                gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(text_x, text_y));
			   
			    y += height;
		    }

            String[][] cols = this.parser.WeekColsParsing(_CurrentPage);
		    width = (this.pageWidth - this.leftScaleWidth) / cols[0].Length;
		    x = this.offsetLeft + this.leftScaleWidth;
		    y = this.offsetTop + this.headerHeight;
		    for (int scale = 0; scale < cols.Length; scale++) {
                for (int col = 0; col < cols[scale].Length; col++)
                {
                    var points = new XPoint[2];
                    points[0] = new XPoint(x, y);
                    points[1] = new XPoint(x, this.pageHeight + this.offsetTop);

                    gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points[0], points[1]);

                    x += width;
                }
                y += headerHeight;
		    }

             var points2 = new XPoint[2];
                points2[0] = new XPoint(x, y - this.multiHeight);
                points2[1] = new XPoint(x, this.offsetTop + this.pageHeight);
               
             gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), points2[0], points2[1]);
        }

        private void weekEventsDraw(PDFPage page)
        {
            double[] eventBorderColor = RGBColor.GetColor(this.eventBorderColor);
            double[] eventColor = RGBColor.GetColor(this.eventColor);

            var headPadding = (double)f1.FontFamily.GetCellAscent(f1.Style) / (double)f1.FontFamily.GetLineSpacing(f1.Style);
            double[] textColor = RGBColor.GetColor(this.eventTextColor);
            var textFormatter = new DHXTextFormatter(gfx);
            var events = page.Events;
            var defAlign = textFormatter.Alignment;

            

            for (int i = 0; i < events.Count; i++)
            {
                double x = this.offsetLeft + this.leftScaleWidth + events[i].getX()
                        * this.weekDayWidth / 100;
                double y = this.offsetTop + this.headerHeight + events[i].getY()
                        * this.weekDayHeight / 100;
                double width = events[i].getWidth() * this.weekDayWidth / 100;
                double height = events[i].getHeight() * this.weekDayHeight / 100;
                String text = events[i].getText();
                String headerText = events[i].getHeaderText();
                String bgColor = events[i].getBackgroundColor();
                String color = events[i].getColor();
                

                var borders = new XPoint[4];
                borders[0] = new XPoint(x, y);
                borders[1] = new XPoint(x + width, y);
                borders[2] = new XPoint(x + width, y + height);
                borders[3] = new XPoint(x, y + height);


                XRect eventBg = new XRect();
                eventBg.Location = new XPoint(x, y);
                eventBg.Size = new XSize(width, height);
                XBrush evBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(bgColor, eventColor)));

                this.f1 = new XFont(f1.FontFamily.Name, BaseFontSize);

                string headerTxt = headerText;
                double text_x = x + (width - gfx.MeasureString(headerTxt, f1).Width) / 2;
                double text_y = y + (this.weekEventHeaderHeight - this.f1.Size) / 2;
                XBrush headTxtBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));

                string bodyTxt = text;
                double body_text_x = x;
                double body_text_width = width;
                if (width > 3 * this.cellOffset) {             
                    body_text_x = x + this.cellOffset;
                    body_text_width = width - 2 * this.cellOffset;
                }
                double body_text_y = y;
                double body_text_height = height;
                if (height > 3 * this.cellOffset)
                {
                    body_text_y =  y + this.cellOffset + this.weekEventHeaderHeight;
                    body_text_height = height - 2 * this.cellOffset;
                }

                
                
                XBrush bodyTxtBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));

                gfx.DrawRectangle(evBrush, eventBg);
                XPen borderPen = new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth);
                gfx.DrawLine(borderPen, borders[0], borders[1]);
                gfx.DrawLine(borderPen, borders[1], borders[2]);
                gfx.DrawLine(borderPen, borders[2], borders[3]);
                gfx.DrawLine(borderPen, borders[3], borders[0]);

                gfx.DrawLine(borderPen, new XPoint(x, y + this.weekEventHeaderHeight), new XPoint(x + width, y + this.weekEventHeaderHeight));
                
                textFormatter.Alignment = XParagraphAlignment.Center;
                headerText = textWrap(headerText, width, f1, this.TrimContentStrings);
                textFormatter.DrawString(headerTxt, f1, headTxtBrush, new XRect(x, text_y, width, this.weekEventHeaderHeight ));
                
                textFormatter.Alignment = defAlign;

                textFormatter.DrawString(bodyTxt, f1, bodyTxtBrush, new XRect(body_text_x, body_text_y, body_text_width, body_text_height));

            }

            // preparing scheme to calculate multiday position
            String[][] cols = this.parser.WeekColsParsing(_CurrentPage);
            int[] scheme = new int[cols[0].Length];
            for (int i = 0; i < scheme.Length; i++)
                scheme[i] = 0;

            events = page.Multiday;
            double offset = 1;
            for (int i = 0; i < events.Count; i++)
            {
                SchedulerEvent ev = events[i];
                int day = ev.getDay();
                int len = ev.getLen();
                String text = ev.getText();
                String bgColor = ev.getBackgroundColor();
                String color = ev.getColor();

                double width = len * this.weekDayWidth - 2 * offset;
                double height = this.monthEventHeight;
                double x = this.offsetLeft + this.leftScaleWidth + day
                        * this.weekDayWidth + offset;
                double y = this.offsetTop + this.headHeight + scheme[day]
                        * this.multidayLineHeight;



                XBrush eventBrush = new XSolidBrush(RGBColor.GetXColor(eventBorderColor));

                XRect cont = new XRect(x, y, width, height);
                XBrush evBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(bgColor, eventColor)));


                this.f1 = new XFont(f1.FontFamily.Name, BaseFontSize);
                string txt = text;//this.textWrap(text, width, this.f1);
                double txt_x = x + this.cellOffset;
                double txt_y = y + (this.weekEventHeaderHeight - this.f1.Size) / 2;
                XBrush textBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));


                gfx.DrawRectangle(evBrush, cont);
                var borders = new XPoint[4];
                borders[0] = new XPoint(x, y);
                borders[1] = new XPoint(x + width, y);
                borders[2] = new XPoint(x + width, y + height);
                borders[3] = new XPoint(x, y + height);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[0], borders[1]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[1], borders[2]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[2], borders[3]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[3], borders[0]);
                //gfx.DrawString(txt, f1, textBrush, new XPoint(txt_x, txt_y));
                textFormatter.DrawString(txt, f1, textBrush, new XRect(txt_x, txt_y, width, height));
                for (int j = day; j < day + len; j++)
                    scheme[j]++;
            }
        }

	    private void weekBwBordersDraw(){
		    double[] borderColor = RGBColor.GetColor(this.lineColor);

		    double x = this.offsetLeft;
		    double y = this.offsetTop;
		    this.Line(borderColor, x + this.leftScaleWidth, y, x + this.pageWidth,
				    y);

		    if (this.multiHeight > 0) {
			    y = this.offsetTop + this.headerHeight - this.multiHeight;
			    this.Line(borderColor, x + this.leftScaleWidth, y, x
					    + this.pageWidth, y);
			    this.Line(borderColor, x, y, x, y + this.pageHeight
					    - (this.headerHeight - this.multiHeight));
		    }

		    y = this.offsetTop + this.headerHeight;
		    this.Line(borderColor, x, y, x + this.pageWidth, y);

		    y = this.offsetTop + this.pageHeight;
		    this.Line(borderColor, x, y, x + this.pageWidth, y);

		    x = this.offsetLeft + this.leftScaleWidth;
		    y = this.offsetTop;
		    this.Line(borderColor, x, y, x, y + this.headerHeight);

		    x = this.offsetLeft + this.pageWidth;
		    this.Line(borderColor, x, y, x, y + this.headerHeight);

	    }

	    private void Line(double[] color, double x1, double y1, double x2, double y2){
            gfx.DrawLine(new XPen(RGBColor.GetXColor(color), BorderWidth), new XPoint(x1,y1), new XPoint(x2,y2));
		   
	    }
        private void Line(double[] color, double x1, double y1, double x2, double y2, XRect cell){
            gfx.DrawLine(new XPen(RGBColor.GetXColor(color), BorderWidth), new XPoint(cell.Location.X + x1,cell.Location.Y + y1), new XPoint(cell.Location.X + x2,cell.Location.Y + y2));
		   
	    }




        private void _drawTimelineContainerPart(string[] rows)
        {
            double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
            double x = this.offsetLeft;
            double y = this.offsetTop + this.headerHeight;
            double width = this.leftScaleWidth;
            double height = (this.pageHeight - this.headerHeight) / rows.Length;
            for (int i = 0; i < rows.Length; i++)
            {

              
                XRect cell = new XRect(x, y, width, height);
                XBrush cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));

                XRect scaleOne = new XRect(x + width, y, this.pageWidth - width, height);
                XBrush scaleOneBrush = new XSolidBrush(RGBColor.GetXColor(scaleOneColor));

                gfx.DrawRectangle(cellBrush, cell);
                gfx.DrawRectangle(scaleOneBrush, scaleOne);

                this.Line(headerLineColor, x, y, x + width, y);

                string text = this.textWrap(rows[i], width - 2 * this.cellOffset, this.f1, this.TrimLayoutStrings);
                double text_x = x + (width - gfx.MeasureString(text, f1).Width) / 2;
                double text_y = y + (height + this.f1.Size) / 2;
                gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(text_x, text_y));

                y += height;
            }
        }


        private double _getPadding(TreeTimelineCategory row)
        {
            double levelPadding = 0.0;
            for (var lev = 0; lev < row.Level; lev++)
            {
                levelPadding += gfx.MeasureString("ww", f1).Width;
            }
            if (row.IsExpandable)
                levelPadding -= gfx.MeasureString("+ ", f1).Width;
            if (levelPadding < 0)
                levelPadding = 0.0;
            return levelPadding;
        }
        private void _drawTreeLevelScales(string[] rows)
        {
            double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
            double x = this.offsetLeft;
            double y = this.offsetTop + this.headerHeight;
            double width = this.leftScaleWidth;
            double height = (this.pageHeight - this.headerHeight) / rows.Length;
            var pars = new TreeTimelineParser();
            f1 = new XFont(f1.FontFamily.Name, f1.Size, XFontStyle.Bold);
            for (int i = 0; i < rows.Length; i++)
            {
                
                var row = pars.Parse(rows[i]);
                if (row.IsExpandable)
                {
                    double levelPadding = _getPadding(row);
                    XRect scaleOne = new XRect(x + width, y, this.pageWidth - width, height);
                    XBrush scaleOneBrush = new XSolidBrush(RGBColor.GetXColor(timelineTreeColor));
                
                   
                    gfx.DrawRectangle(scaleOneBrush, scaleOne);
                    

                    double text_x = levelPadding + x;
                    double text_y = y + (height + this.f1.Size) / 2;
                    var text = (row.Expanded ? " - " : " + ") + row.Text;
                    text = this.textWrap(text, this.pageWidth, this.f1, this.TrimLayoutStrings);
                    gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(text_x, text_y));
                }
                y += height;
            }
            f1 = new XFont(f1.FontFamily.Name, f1.Size, XFontStyle.Regular);
        }
        private void _drawTreeTimelineContainerPart(string[] rows)
        {
            double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
            double x = this.offsetLeft;
            double y = this.offsetTop + this.headerHeight;
            double width = this.leftScaleWidth;
            double height = (this.pageHeight - this.headerHeight) / rows.Length;

            var pars = new TreeTimelineParser();

            for (int i = 0; i < rows.Length; i++)
            {
                x = this.offsetLeft;
                var row = pars.Parse(rows[i]);

                double levelPadding = _getPadding(row);
               

                XRect cell = default(XRect);
                XBrush cellBrush = null;

                XRect scaleOne = default(XRect);
                XBrush scaleOneBrush = null;

                

                if (row.IsExpandable)
                {
                    f1 = new XFont(f1.FontFamily.Name, f1.Size, XFontStyle.Bold);
                    cell = new XRect(x, y, width, height);
                    cellBrush = new XSolidBrush(RGBColor.GetXColor(timelineTreeColor));
                    gfx.DrawRectangle(cellBrush, cell);

                }else{
                    f1 = new XFont(f1.FontFamily.Name, f1.Size, XFontStyle.Regular);

                    cell = new XRect(x, y, width, height);
                    cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));

                    scaleOne = new XRect(x + width, y, this.pageWidth - width, height);
                    scaleOneBrush = new XSolidBrush(RGBColor.GetXColor(scaleOneColor));

                    string text = "";
                    text = this.textWrap(row.Text, width - 2 * this.cellOffset - levelPadding, this.f1, this.TrimLayoutStrings);
                    double text_x = levelPadding + x ;
                    double text_y = y + (height + this.f1.Size) / 2;
                    gfx.DrawRectangle(cellBrush, cell);
                    gfx.DrawRectangle(scaleOneBrush, scaleOne);
                    gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(text_x, text_y));


                }
  
                this.Line(headerLineColor, x, y, x + width, y);


                
                y += height;
            }

        }

	    private void timelineContainerDraw(){
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
		    double[] scaleOneColor = RGBColor.GetColor(this.scaleOneColor);
            string[] rows = this.parser.WeekRowsParsing(_CurrentPage);
		    double width = this.leftScaleWidth;
		    double height = (this.pageHeight - this.headerHeight) / rows.Length;
		    this.weekDayHeight = height;
		    double x = this.offsetLeft;
		    double y = this.offsetTop + this.headerHeight;

            bool isTree = false;
            if (rows.Length > 0)
            {
                isTree = TreeTimelineParser.IsTreeRow(rows[0]);
            }
            
            if (isTree)
            {
                _drawTreeTimelineContainerPart(rows);
            }else{
                _drawTimelineContainerPart(rows);
            }



            String[][] cols = this.parser.WeekColsParsing(_CurrentPage);
            width = double.MaxValue;//(this.pageWidth - this.leftScaleWidth) / cols[0].Length;
            var colCount = -1;
            for (var i = 0; i < cols.Length; i++)
            {
                var scale = (this.pageWidth - this.leftScaleWidth) / cols[i].Length;
                if (width > scale)
                    width = scale;
                if (colCount < cols[i].Length)
                    colCount = cols[i].Length;
            }

		    x = this.offsetLeft + this.leftScaleWidth;
		    y = this.offsetTop + this.headerHeight;
            for (int i = 0; i < colCount; i++)
            {
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), 
                    new XPoint(x, y), new XPoint(x, this.pageHeight + this.offsetTop));

			    x += width;
		    }

            if (isTree)
                _drawTreeLevelScales(rows);
         
            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth),
                new XPoint(this.offsetLeft + this.leftScaleWidth, y), new XPoint(this.offsetLeft + this.leftScaleWidth + width, y));
              
		    y = this.offsetTop + this.headerHeight + height;
		    for (int i = 0; i < rows.Length; i++) {
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), 
                    new XPoint(this.offsetLeft + this.leftScaleWidth, y), new XPoint(this.offsetLeft + this.pageWidth, y));
			    y += height;
		    }

		    

            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), 
                new XPoint(this.offsetLeft + this.leftScaleWidth, this.offsetTop + this.headerHeight), 
                new XPoint(this.offsetLeft + this.pageWidth, this.offsetTop + this.headerHeight));

            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), 
                new XPoint(x, this.offsetTop), 
                new XPoint(x, this.offsetTop + this.pageHeight));
	    }

	    private void matrixContainerDraw(PDFPage page) {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
		    double[] matrixEventColor = RGBColor.GetColor(this.matrixEventColor);
		    double[] textColor = RGBColor.GetColor(this.textColor);
            String[] rows = this.parser.WeekRowsParsing(_CurrentPage);
		    double height = (this.pageHeight - this.headerHeight) / rows.Length;
            String[][] cols = this.parser.WeekColsParsing(_CurrentPage);
		    var events = page.Events;
            var columns = cols[0];
		    for (int i = 0; i < rows.Length; i++) {
                for (int j = 0; j <= columns.Length; j++)
                {
                    SchedulerEvent ev = events[i * (columns.Length + 1) + j];
				    String evBgColor = ev.getBackgroundColor();
				    String evTextColor = ev.getColor();
				    String text = ev.getText();
				    double x = this.offsetLeft + Math.Max(j - 1, 0)
						    * this.weekDayWidth + (j != 0 ? 1 : 0)
						    * this.leftScaleWidth;
				    double y = this.offsetTop + this.headerHeight + i * height;

				    double width = (j == 0) ? this.leftScaleWidth : this.weekDayWidth;


                    XRect cell = new XRect(x, y, width, height);
				    XBrush cellBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(evBgColor, matrixEventColor)));


				    
				    if (j == 0)
					    cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));
                    gfx.DrawRectangle(cellBrush, cell);
				   

				    // draw cell text
                    this.f1 = new XFont(f1.FontFamily.Name, (j == 0) ? BaseFontSize : 8.4);
				    string txt = this.textWrap(text, width - 2 * this.cellOffset, this.f1, this.TrimContentStrings);
				    double text_x = x+ (width - gfx.MeasureString(txt, f1).Width) / 2;
				    double text_y = y+(height - this.f1.Size) / 2;
				    //txt.setPosition(text_x, text_y);
                    XBrush txtBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(evTextColor, textColor)));
				    
                    gfx.DrawString(txt, f1, txtBrush, new XPoint(text_x, text_y));
				    

				    // draw borders
				    if (j == 0) {
					    this.Line(headerLineColor, x, y, x + width, y);
				    } else {
					    this.Line(borderColor, x, y, x + width, y);
					    this.Line(borderColor, x, y, x, y + height);
				    }
			    }
		    }

		    // border right
            this.Line(borderColor, this.offsetLeft + this.pageWidth, this.offsetTop, this.offsetLeft + this.pageWidth, this.offsetTop
				    + this.pageHeight);

            this.Line(borderColor, this.offsetLeft, this.offsetTop + this.pageHeight, this.offsetLeft + this.pageWidth, this.offsetTop
				    + this.pageHeight);
	    }

	    private void timelineEventsDraw(PDFPage page) {
		    double[] eventBorderColor = RGBColor.GetColor(this.eventBorderColor);
		    double[] eventColor = RGBColor.GetColor(this.eventColor);
		    double[] textColor = RGBColor.GetColor(this.eventTextColor);
		    var events = page.Events;
           
		    double offset = 1;
		    for (int i = 0; i < events.Count; i++) {
			    SchedulerEvent ev = events[i];
			    String text = ev.getText();
			    String bgColor = ev.getBackgroundColor();
			    String color = ev.getColor();

			    double x = this.offsetLeft + this.leftScaleWidth + ev.getX()
					    * this.weekDayWidth / 100 + offset;
			    double y = this.offsetTop + this.headerHeight + ev.getY()
					    * this.weekDayHeight / 100 + ev.getWeek()
					    * this.weekDayHeight;
			    double width = ev.getWidth() * this.weekDayWidth / 100;
			    double height = this.monthEventHeight;


                XRect cell = new XRect(x, y, width, height);
			   
				XBrush cellBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(bgColor, eventColor)));



                this.f1 = new XFont(f1.FontFamily.Name, BaseFontSize);
			    string txt = this.textWrap(text, width,	this.f1, this.TrimContentStrings);
			    double txt_x = x+this.cellOffset;
			    double txt_y = y+(this.weekEventHeaderHeight + this.f1.Size) / 2;
                XBrush txtBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));
			    

                gfx.DrawRectangle(cellBrush, cell);

			    var borders = new XPoint[4];
                borders[0] = new XPoint(x, y);
                borders[1] = new XPoint(x + width, y);
                borders[2] = new XPoint(x + width, y + height);
                borders[3] = new XPoint(x, y + height);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[0], borders[1]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[1], borders[2]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[2], borders[3]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth), borders[3], borders[0]);

                gfx.DrawString(txt, f1, txtBrush, new XPoint(txt_x, txt_y));
		    }
	    }

	    private void yearDraw(PDFPage page) {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
		    double[] textColor = RGBColor.GetColor(this.textColor);
		    double[] eventColor = RGBColor.GetColor(this.eventColor);
		    double[] yearDayActiveColor = RGBColor.GetColor(this.yearDayActiveColor);
		    double[] yearDayInactiveColor = RGBColor.GetColor(this.yearDayInactiveColor);
            SchedulerMonth[] monthes = this.parser.YearParsing(_CurrentPage);
		    var events = page.Events;
		    double width = (this.pageWidth - this.yearMonthOffsetLeft * 3) / 4;
		    double height = (this.pageHeight - this.yearMonthOffsetTop * 2) / 3;
		    double month_x = this.offsetLeft;
		    double month_y = this.offsetTop;
		    for (int i = 0; i < 3; i++) {
			    for (int j = 0; j < 4; j++) {
				    SchedulerMonth mon = monthes[i * 4 + j];
				    String label = mon.GetLabel();
				    String[,] rows = mon.GetRows();
				    String[,] onlyDays = mon.GetOnlyDays();

                    XRect monthCont = new XRect();
			        monthCont.Location = new XPoint(month_x, month_y);
                    monthCont.Size = new XSize(width, height);
				    XBrush monthContBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));


				    string labelText = this.textWrap(label,	 width, this.f1, this.TrimLayoutStrings);
				    double label_text_x = month_x + (width - gfx.MeasureString(label, f1).Width) / 2;
				    double label_text_y =month_y + (this.yearMonthLabelHeight + this.f1.Size) / 2;
				    XBrush labelTextBrush = new XSolidBrush(RGBColor.GetXColor(textColor));
				   
                    gfx.DrawRectangle(monthContBrush, monthCont);
                    gfx.DrawString(labelText, f1, labelTextBrush, new XPoint(label_text_x, label_text_y));
				  

				    double cell_width = width / 7;
				    double cell_height = (height - this.yearMonthLabelHeight)
						    / rows.GetLength(0);
                    double cell_x = month_x;
				    double cell_y = month_y+ this.yearMonthLabelHeight;
				    for (int k = 0; k < rows.GetLength(0); k++) {
					    for (int l = 0; l < 7; l++) {

                            XRect cell = new XRect();
                            cell.Location = new XPoint(cell_x, cell_y);
                            cell.Size = new XSize(cell_width, cell_height);
				            XBrush cellBrush;

						    int ind = this.getEventIndex(events, l, k - 1, i * 4 + j);
						    if (k == 0) {
							    cellBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));
						    } else {
							    if (this.getActiveDay(onlyDays, k - 1, l) == true) {
								    if (ind == -1) {
                                        cellBrush = new XSolidBrush(RGBColor.GetXColor(yearDayActiveColor));
								    } else {
									    SchedulerEvent ev = events[ind];
									    String color = ev.getBackgroundColor();
                                        cellBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, eventColor)));
									    
								    }
							    } else {
                                    cellBrush = new XSolidBrush(RGBColor.GetXColor(yearDayInactiveColor));
		
							    }
						    }
                            gfx.DrawRectangle(cellBrush, cell);
						    
						    if (k == 0)
							    this.Line(headerLineColor, 0, 0, cell_width, 0,
									    cell);
						    else
							    this.Line(borderColor, 0, 0, cell_width, 0, cell);
						    if (l > 0)
							    if (k == 0)
								    this.Line(headerLineColor, 0, 0, 0,
										    cell_height, cell);
							    else
								    this.Line(borderColor, 0, 0, 0, cell_height,
										    cell);

						    string cell_text =  rows[k,l];
                            double cell_text_x = cell_x + (cell_width - gfx.MeasureString(cell_text, f1).Width) / 2;
                            double cell_text_y = cell_y + (cell_height + this.f1.Size) / 2;
						    XSolidBrush evBrush;

						    if (ind > -1) {
							    SchedulerEvent ev = events[ind];
							    String color = ev.getColor();
                                evBrush = new XSolidBrush(RGBColor.GetXColor(selectColor(color, textColor)));
							   
						    } else
							    evBrush = new XSolidBrush(RGBColor.GetXColor(textColor));
                            gfx.DrawString(cell_text, f1, evBrush, new XPoint(cell_text_x, cell_text_y));
						    cell_x += cell_width;
					    }
                        cell_x = month_x; 
					    cell_y += cell_height;
				    }
				    if (this.profile == ColorProfile.BW) {
					    this.Line(headerLineColor, 0, 0, 0, height, monthCont);
					    this.Line(headerLineColor, 0, 0, width, 0, monthCont);
					    this.Line(headerLineColor, 0, height, width, height,
							    monthCont);
					    this.Line(headerLineColor, width, 0, width, height,
							    monthCont);
				    }
				    month_x += this.yearMonthOffsetLeft + width;
			    }
			    month_x = this.offsetLeft;
			    month_y += this.yearMonthOffsetTop + height;
		    }
	    }

	    private void agendaHeaderDraw()  {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.headerLineColor);
		    double[] textColor = RGBColor.GetColor(this.textColor);
            String[] cols = this.parser.AgendaColsParsing(_CurrentPage);

		    double width = this.pageWidth;
		    double height = this.headerHeight;
		    double x = this.offsetLeft;
		    double y = this.offsetTop;

            XRect headerBg = new XRect();
			headerBg.Location = new XPoint(x, y);
            headerBg.Size = new XSize(width, height);
			XBrush monthContBrush = new XSolidBrush(RGBColor.GetXColor(bgColor));
            gfx.DrawRectangle(monthContBrush, headerBg);
		  
		    //headerBg.placeIn(headerCont, 0, 0);

		    double date_width = this.agendaColOneWidth;
		    double name_width = width - this.agendaColOneWidth;

		    var sep = new XPoint[2]{new XPoint(x + date_width, y), new XPoint(x + date_width, y + this.headerHeight)};
            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), 0.3), sep[0], sep[1]);


            this.f1 = new XFont(f1.FontFamily.Name, BaseFontSize);
            string dateText = cols[0];

		    XBrush textBrush = new XSolidBrush(RGBColor.GetXColor(textColor));
		    double date_text_x = x + (date_width - gfx.MeasureString(dateText, f1).Width) / 2;
		    double date_text_y = y + (height + this.f1.Size) / 2;
            gfx.DrawString(dateText, f1, textBrush, new XPoint(date_text_x, date_text_y));

		   

		    string nameText = cols[1];
		    double name_text_x = x + date_width + (name_width - gfx.MeasureString(nameText, f1).Width) / 2;
		    double name_text_y = y + (height + this.f1.Size) / 2;
		    gfx.DrawString(nameText, f1, textBrush, new XPoint(name_text_x, name_text_y));

		    
            var borders = new XPoint[4]{
                new XPoint(x, y), 
                new XPoint(x + width, y), 
                new XPoint(x + width, y + height), 
                new XPoint(x, y + height)};
            gfx.DrawLine(new XPen(RGBColor.GetXColor(lineColor), 0.3), borders[0], borders[1]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(lineColor), 0.3), borders[1], borders[2]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(lineColor), 0.3), borders[2], borders[3]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(lineColor), 0.3), borders[3], borders[0]);

	    }

	    private void agendaEventDraw(PDFPage page)  {
		    double[] scaleOneColor = RGBColor.GetColor(this.scaleOneColor);
		    double[] scaleTwoColor = RGBColor.GetColor(this.scaleTwoColor);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] textColor = RGBColor.GetColor(this.textColor);
            var events = page.Events;
		    double width = this.pageWidth;
		    double height = this.headerHeight;
		    double x = this.offsetLeft;
		    double y = this.offsetTop + this.headerHeight;

            var linePen = new XPen(RGBColor.GetXColor(eventBorderColor), BorderWidth);

            this.f1 = new XFont(f1.FontFamily.Name, BaseFontSize);
		    for (int i = 0; i < events.Count; i++) {
			    if (y + height > this.offsetTop + this.pageHeight) {

                    gfx.DrawLine(linePen,new XPoint(x,y), new XPoint(x + width, y));

				  
				    this.page = newPage(PageOrientation.Portrait, PageSize.A4);
				 
				    this.agendaHeaderDraw();
				    y = this.offsetTop + this.headerHeight;
			    }
			   


                var headerCont = new XRect(x, y, width, height);
			    XBrush headerBrush;
			    //headerCont.setPattern(this.linePattern);
			    if (i % 2 == 0) {
				    headerBrush = new XSolidBrush(RGBColor.GetXColor(scaleOneColor));
			    } else {
				    headerBrush = new XSolidBrush(RGBColor.GetXColor(scaleTwoColor));
			    }

			    double date_width = this.agendaColOneWidth;

                gfx.DrawRectangle(headerBrush, headerCont);

                var borders = new XPoint[4]{
                    new XPoint(x, y), 
                    new XPoint(x + date_width, y), 
                    new XPoint(x + date_width, y + height), 
                    new XPoint(x, y + height)};
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[0], borders[1]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[1], borders[2]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[2], borders[3]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[3], borders[0]);

                string headerText = events[i].getHeaderAgendaText();
                string text = events[i].getText();

                XSolidBrush textBrush = new XSolidBrush(RGBColor.GetXColor(textColor));

                double date_text_x = x + (date_width - gfx.MeasureString(headerText, f1).Width) / 2;
			    double date_text_y = y + (height + this.f1.Size) / 2;
                gfx.DrawString(headerText, f1, textBrush, new XPoint(date_text_x, date_text_y));
			   

			    
			    double name_text_x = x + date_width + this.cellOffset;
			    double name_text_y = y + (height + this.f1.Size) / 2;
                gfx.DrawString(text, f1, textBrush, new XPoint(name_text_x, name_text_y));
			  


                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), new XPoint(x,y), new XPoint(x, y+height));
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), new XPoint(x,y), new XPoint(x + width, y));
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), new XPoint(x+width,y), new XPoint(x + width, y+height));
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), new XPoint(x+date_width,y), new XPoint(x + date_width, y+height));

			   
			    y += height;
		    }

            gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), new XPoint(x,y), new XPoint(x + width, y));
               
		
	    }

	    private void agendaPagesDraw(){
            for (var i = 1; i <= pages.Count; i++)
            {
                var page = pages[i-1];
                var graph = graphics[i-1]; 
                 if ((this.profile == ColorProfile.BW)
                        && (this._CurrentPage.Mode != "month"))
				    this.monthBwBordersDraw();
			    String str = this.pageNumTemplate;
                str = str.Replace("{pageNum}", i.ToString());
			    str = str.Replace("{allNum}", pages.Count.ToString());

                XTextFormatter text = new XTextFormatter(graph);
                double x = pageWidth + offsetLeft - graph.MeasureString(str, f1).Width;
			    double y = pageHeight + offsetTop + f1.Size;
                text.DrawString(str, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XRect(x, y, graph.MeasureString(str, f1).Width, f1.Size));
		    }

	    }


	    private void weekAgendaContainerDraw() {
            string[] cols = this.parser.AgendaColsParsing(_CurrentPage);
		    double x = this.offsetLeft;
		    double contWidth = (this.pageWidth)/2;
		    double contHeight = (this.pageHeight)/3;
		    this.contWidth = contWidth;
		    this.contHeight = contHeight;
		
		
		    for (int i = 0; i < 3; i++) {
			    double y = this.offsetTop + contHeight*i;
			    this.weekAgendarDayDraw(cols[i], contWidth, contHeight, x, y);
		    }
		    x += contWidth;
		    for (int i = 0; i < 2; i++) {
			    double y = this.offsetTop + contHeight*i;
			    this.weekAgendarDayDraw(cols[i + 3], contWidth, contHeight, x, y);
		    }
		
		    double tallContHeight = contHeight/2;
		    for (int i = 0; i < 2; i++) {
			    double y = this.offsetTop + contHeight*2 + tallContHeight*i;
			    this.weekAgendarDayDraw(cols[i + 5], contWidth, tallContHeight, x, y);
		    }
		    double[] headerLineColor = RGBColor.GetColor(this.headerLineColor);
		    for (int i = 0; i < 3; i++) {
			    double y = this.offsetTop + contHeight*i;
			    this.Line(headerLineColor, x, y, x, y + this.monthDayHeaderHeight);
		    }
	    }

	    private void weekAgendarDayDraw(String name, double width, double height, double x, double y)  {
		    double[] bgColor = RGBColor.GetColor(this.bgColor);
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] textColor = RGBColor.GetColor(this.textColor);
            this.f1 = new XFont(f1.FontFamily.Name, WeekAgendarDayFont);
            var day_cont = new XRect(x, y, width, height);
		    //Box day_cont = new Box();

            var borders = new XPoint[4]{
                new XPoint(x, y), 
                new XPoint(x + width, y), 
                new XPoint(x + width, y + height), 
                new XPoint(x, y + height)};
            gfx.DrawLine(new XPen(RGBColor.GetXColor(bgColor), BorderWidth), borders[0], borders[1]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(bgColor), BorderWidth), borders[1], borders[2]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(bgColor), BorderWidth), borders[2], borders[3]);
            gfx.DrawLine(new XPen(RGBColor.GetXColor(bgColor), BorderWidth), borders[3], borders[0]);

		    this.Line(borderColor, 0, 0, width, 0, day_cont);
		    this.Line(borderColor, 0, 0, 0, height, day_cont);
		    this.Line(borderColor, width, 0, width, height, day_cont);
		    this.Line(borderColor, 0, height, width, height, day_cont);

            var label_cont = new XRect(x,y,width,monthDayHeaderHeight);
            gfx.DrawRectangle(new XSolidBrush(RGBColor.GetXColor(bgColor)), label_cont);
          
		    var txt = name;
		    x = x + this.cellOffset + (width - cellOffset - gfx.MeasureString(name, f1).Width)/2;
		    y = y + (this.monthDayHeaderHeight + this.f1.Size) / 2;
		    gfx.DrawString(txt, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(x,y));
	    }

	    private SchedulerEvent[] weekAgendaEventsDraw(SchedulerEvent[] events) {
		    double[] borderColor = RGBColor.GetColor(this.lineColor);
		    double[] textColor = RGBColor.GetColor(this.textColor);
		    int[] offsets = new int[7];
		    for (int i = 0; i < offsets.Length; i++)
			    offsets[i] = 0;
		    var rest = new List<SchedulerEvent>(); 

		    for (int i = 0; i < events.Length; i++) {
			    SchedulerEvent ev = events[i];
			    int day = ev.getDay();
			    double cont_height = (day < 5) ? this.contHeight : this.contHeight/2;
			    double cont_width = this.contWidth;
			    double x;
			    switch (day) {
				    case 0:
				    case 2:
				    case 4:
					    x = this.offsetLeft;
					    break;
				    default:
					    x = this.offsetLeft + cont_width;
					    break;
			    }
			    double cont_start_y = this.offsetTop + Math.Floor(day/2.0)*this.contHeight - (day > 5 ? cont_height : 0);
			    double offset = offsets[day]*this.weekAgendaEventHeight;
			    double y = cont_start_y + this.monthDayHeaderHeight+ offset;

			    if (cont_start_y + cont_height < y + this.weekAgendaEventHeight) {
				    rest.Add(ev);
				    continue;
			    }


                var borders = new XPoint[4]{
                    new XPoint(x, y), 
                    new XPoint(x + contWidth, y), 
                    new XPoint(x + contWidth, y + weekAgendaEventHeight), 
                    new XPoint(x, y + weekAgendaEventHeight)};
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[0], borders[1]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[1], borders[2]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[2], borders[3]);
                gfx.DrawLine(new XPen(RGBColor.GetXColor(borderColor), BorderWidth), borders[3], borders[0]);
			  

			    this.f1 = new XFont(f1.FontFamily.Name, 9);
			    String text = ev.getText();
			 
			    x = x + this.cellOffset;
			    y = y + (this.weekAgendaEventHeight +  this.f1.Size) / 2;
                gfx.DrawString(text, f1, new XSolidBrush(RGBColor.GetXColor(textColor)), new XPoint(x,y));
			   
			    offsets[day]++;
		    }
		    SchedulerEvent[] events_list = new SchedulerEvent[rest.Count];
		    for (int i = 0; i < rest.Count; i++)
			    events_list[i] = rest[i];
		    return events_list;
	    }

	    private void todayLabelDraw(){
            var g1 = this.graphics[0];
		    this.todayLabelDraw(g1);
	    }

	    private void todayLabelDraw(XGraphics p1)  {
          
		    this.f1 = new XFont(f1.FontFamily.Name, 10);
            
		    String today = this._CurrentPage.TodayLabel;
		    var todayText = today;
		    double today_x = this.offsetLeft;
		    double today_y = this.offsetTop - this.cellOffset;
            p1.DrawString(todayText, f1, new XSolidBrush(RGBColor.GetXColor(this.textColor)), new XPoint(today_x, today_y));

	    }
	
	    private int getEventIndex(IList<SchedulerEvent> events, int day, int week, int month){
		    for (int i = 0; i < events.Count; i++) {
			    int ev_day = events[i].getDay();
			    int ev_week = events[i].getWeek();
			    int ev_month = events[i].getMonth();
			    if ((ev_day == day) && (ev_week == week) && (ev_month == month)) {
				    return i;
			    }
		    }
		    return -1;
	    }




	    private bool getActiveDay(string[,] rows, int row, int col) {
		    bool flag = true;
		    int flagCount = 0;
		    if (int.Parse(rows[0,0]) == 1) {
			    flag = true;
			    flagCount = 1;
		    } else {
			    flag = false;
		    }

		    int prevDay = int.Parse(rows[0,0]);
		    for (int i = 0; i < rows.Length; i++) {
			    for (int j = 0; j < rows.GetLength(1); j++) {
				    if ((int.Parse(rows[i, j]) < prevDay) && (flagCount < 2)) {
					    flag = !flag;
					    flagCount++;
				    }
				    if ((i == row) && (j == col)) {
					    return flag;
				    }
				    prevDay = int.Parse(rows[i, j]);
			    }
		    }
		    return flag;
	    }

	    public string getView() {
		    return this.view;
	    }



        private void printHeader()
        {
           
            if (this._CurrentPage.Header == true)
            {
                XImage im = images.Get(HeaderImgPath);
                if (im != null)
                {
                    gfx.DrawImage(im, new XPoint(offsetLeft, offsetTop));
                    pageHeight -= HeaderImgHeight;
                    offsetTop += HeaderImgHeight;
                }
            }
        }

        private void printFooter()
        {

            if (this._CurrentPage.Footer == true)
            {
                XImage im = images.Get(FooterImgPath);
                if (im != null)
                {
                    gfx.DrawImage(im, new XPoint(offsetLeft, pageHeight + offsetTop - FooterImgHeight));
                    pageHeight -= FooterImgHeight;
                    offsetBottom += FooterImgHeight;
                }
            }
        }
        

	    public void SetWatermark(string mark) {
		    watermark = mark;	
	    }

        

    }
}
