﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
namespace DHTMLX.Export.PDF.Scheduler
{
    public enum ColorProfile
    {
        Color,
        FullColor,
        Gray,
        BW,
        Custom
    }



    public class XMLParser
    {
        private string xml;
	    private XmlNode root;

        public List<PDFPage> Pages { get; set; }
	

        public XMLParser()
        {
            Pages = new List<PDFPage>();
        }

        public ColorProfile StringToColorProfile(string profile)
        {
            if (profile == null)
                return ColorProfile.Color;
            switch (profile.ToLower())
            {
                case "gray":
                    {
                        return ColorProfile.Gray;

                    }
                case "full_color":
                    {
                        return ColorProfile.FullColor;
                    }
                case "color":
                    {
                        return ColorProfile.Color;
                    }
                case "bw":
                    {
                        return ColorProfile.BW;
                    }
                case "custom":{
                    return ColorProfile.Custom;
                }
                default:
                    return ColorProfile.Color;

            }
        }

        public static List<XmlNode> GetElementsByTagName(XmlNode parent, string name)
        {
            var list = new List<XmlNode>();
            foreach (XmlNode node in parent.ChildNodes)
            {
                if (node.LocalName == name)
                {
                    list.Add(node);
                }
                else
                {
                    list.AddRange(GetElementsByTagName(node, name));
                }
            }
            return list;
        }

        public void ParsePages(XmlNode root)
        {

            if (root.Name == "pages")
            {
                var n1 = GetElementsByTagName(root, "page");
                if ((n1 != null) && (n1.Count > 0))
                {

                    for (int i = 0; i < n1.Count; i++)
                    {
                        var page = new PDFPage();
                        page.Node = n1[i];
                        this.Pages.Add(page);
                        this.ParseConfig(page);
                        this.EventsParsing(page);
                    }
                }
            }
            else
            {
                var page = new PDFPage();
                page.Node = root;
                this.Pages.Add(page);
                this.ParseConfig( page);
                this.EventsParsing( page);
            }

        }
        public void ParseConfig( PDFPage page)
        {
            XmlNode scale = null;
            var root = page.Node;
            foreach (XmlNode nod in root.ChildNodes)
            {
                if (nod.LocalName == "scale")
                {
                    scale = nod;
                    break;
                }
            }

            var toCheck = new List<XmlNode>();
            toCheck.Add(root);
            if (scale != null)
                toCheck.Add(scale);
            if (root.ParentNode != null && root.ParentNode.NodeType != XmlNodeType.Document)
            {
                toCheck.Add(root.ParentNode);
            }

            for (var i = 0; i < toCheck.Count; i++)
            {
                if (toCheck[i].Attributes["profile"] != null && page.Profile == default(ColorProfile))
                {
                    page.Profile = StringToColorProfile(toCheck[i].Attributes["profile"].Value);
                }
                if (toCheck[i].Attributes["header"] != null && page.Header == default(bool))
                {
                    page.Header = (toCheck[i].Attributes["header"].Value == "true");
                }
                if (toCheck[i].Attributes["footer"] != null && page.Footer == default(bool))
                {
                    page.Footer = (toCheck[i].Attributes["footer"].Value == "true");
                }

                if (toCheck[i].Attributes["mode"] != null && page.Mode == default(string))
                    page.Mode = toCheck[i].Attributes["mode"] != null ? toCheck[i].Attributes["mode"].Value : "";

                if (toCheck[i].Attributes["today"] != null && page.TodayLabel == default(string))
                    page.TodayLabel = toCheck[i].Attributes["today"] != null ? toCheck[i].Attributes["today"].Value : "";
                
            }

          
        }
	    public void SetXML(string xml){
		    this.xml = xml;
            XmlDocument dom = new XmlDocument();          
            dom.LoadXml(xml);

            root = dom.DocumentElement;


            this.ParsePages(root);
	    }

	    public String[][] MonthColsParsing(PDFPage page) {

            var n1 = GetElementsByTagName(page.Node, "column"); 
		    if ((n1 != null)&&(n1.Count > 0)) {
               // page.Cols = new String[n1.Count,1];
                page.Cols = new string[1][]{new string[n1.Count]};
               
                for (int i = 0; i < n1.Count; i++)
                {
				    var col = n1[i];
                    page.Cols[0][i] = col.FirstChild.Value;
			    }
		    }
            return page.Cols;
	    }

        public String[,] MonthRowsParsing(PDFPage page)
        {
            var n1 = GetElementsByTagName(page.Node, "row");
		    if ((n1 != null)&&(n1.Count > 0)) {
			    page.Rows = new string[n1.Count,7];
			    for (int i = 0; i < n1.Count; i++) {
				    var row = n1[i];
				    String week = row.FirstChild.Value;
                    var days = week.Split(new string[1]{"|"}, StringSplitOptions.None );
                    for (var j = 0; j < days.Length; j++)
                        page.Rows[i, j] = days[j];
			    }
		    }
            return page.Rows;
	    }

	    public void EventsParsing(PDFPage page) {
            var n1 = GetElementsByTagName(page.Node, "event");
		    if ((n1 != null)&&(n1.Count > 0)) {
                for (int i = 0; i < n1.Count; i++)
                {
				    var ev = n1[i];
				    SchedulerEvent oEv = new SchedulerEvent();
				    oEv.Parse(ev);
                    if ((oEv.getType() == "event_line") && (page.Mode != "month") && (page.Mode != "timeline") && (page.Mode != "treetimeline"))
                    {
                        page.Multiday.Add(oEv);
				    } else {
                        page.Events.Add(oEv);
				    }
			    }
		    }
	    }

        public string[][] WeekColsParsing(PDFPage page)
        {
		    if (page.Cols != null)
                return page.Cols;
            var n1 = GetElementsByTagName(page.Node, "column");

            if ((n1 != null) && (n1.Count > 0))
            {

                var scale_1 = n1.Where(n=>n.Attributes["second_scale"] == null).ToList();
                var scale_2 = n1.Where(n=>n.Attributes["second_scale"] != null).ToList();

                string[][] scales;
                if (scale_2.Count>0)
                {
                    scales = new string[2][];
                    scales[0] = new string[scale_1.Count];
                    scales[1] = new string[scale_2.Count];
                }
                else
                {
                    scales = new string[1][];
                    scales[0] = new string[scale_1.Count];
                }

                for (var i = 0; i < scale_1.Count; i++)
                {
                    scales[0][i] = scale_1[i].FirstChild.Value;
                }
                int ind = 0;
                for (var i = 0; i < scale_2.Count; i++)
                {
                    //if (int.TryParse(scale_2[i].Attributes["second_scale"].Value, out ind))
                  //      ind--;
                  //  else
                  //      ind = i;
                  //  if( ind < 0 || ind >= scale_1.Count)
                  //      ind = i;
                    scales[1][i] = scale_2[i].FirstChild.Value;
                }
                page.Cols = scales;

		    }
            return page.Cols;
	    }

        public String[] WeekRowsParsing(PDFPage page)
        {
		    String[] rows = null;
            var n1 = GetElementsByTagName(page.Node, "row");
		    if ((n1 != null)&&(n1.Count > 0)) {
                rows = new String[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    var row = n1[i];
				    rows[i] = row.FirstChild.Value;
			    }
		    }
		    return rows;
	    }

        public SchedulerMonth[] YearParsing(PDFPage page)
        {
		    SchedulerMonth[] monthes = null;
            var n1 = GetElementsByTagName(page.Node, "month");
            if ((n1 != null) && (n1.Count > 0))
            {
                monthes = new SchedulerMonth[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    monthes[i] = new SchedulerMonth();
				    var mon = n1[i];
				    monthes[i].Parse(mon);
			    }
		    }
		    return monthes;
	    }

        public String[] AgendaColsParsing(PDFPage page)
        {
		    String[] cols = null;
            var n1 = GetElementsByTagName(page.Node, "column");
            if ((n1 != null) && (n1.Count > 0))
            {
                cols = new String[n1.Count];
                for (int i = 0; i < n1.Count; i++)
                {
				    var col = n1[i];
                    cols[i] = col.FirstChild.Value;
			    }
		    }
		    return cols;
	    }

    }
}
